module Examples where

import Prelude hiding (map, filter, (.), flip, foldr, foldl, sum, concat, length, and, all)
import Data.Char(toLower, toUpper, isAlpha)

toL :: String-> String
toL []     = []
toL (c:cs) = toLower c : toL cs

toU :: String-> String
toU []     = []
toU (c:cs) = toUpper c : toU cs    

map :: (alpha-> beta)-> [alpha]-> [beta]
map f []     = []
map f (c:cs) = f c : map f cs

toL' cs = map toLower cs
toU' cs = map toUpper cs

filter :: (alpha-> Bool)-> [alpha]-> [alpha]
filter p []   = []
filter p (x:xs) 
  | p x       = x: filter p xs
  | otherwise = filter p xs

letters :: String-> String
letters = filter isAlpha    


(.) :: (beta-> gamma) -> (alpha-> beta)-> alpha-> gamma
(f . g) x = f (g x)

(>..>) :: (alpha-> beta)-> (beta-> gamma)-> alpha-> gamma
(f >..> g) x = g (f x)

flip :: (alpha-> beta-> gamma)-> beta-> alpha-> gamma
flip f b a = f a b

(>.>) :: (alpha-> beta)-> (beta-> gamma)-> alpha-> gamma
(>.>) = flip (.)

foldr :: (alpha-> beta-> beta)-> beta-> [alpha]-> beta
foldr f e []     = e
foldr f e (x:xs) = f x (foldr f e xs)

sum :: [Int]-> Int
sum xs = foldr (+) 0 xs

concat :: [[a]]-> [a]
concat xs = foldr (++) [] xs

length :: [a]-> Int
length xs = foldr (\x n-> n+ 1) 0 xs    

and :: [Bool] -> Bool
and xs = foldr (&&) True xs

all :: (alpha-> Bool)-> [alpha] -> Bool
all p = and . map p

foldl :: (alpha -> beta -> alpha) -> alpha -> [beta] -> alpha
foldl f a [] = a
foldl f a (x:xs) = foldl f (f a x) xs
